"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GeneratorBase = void 0;
const fs_1 = require("fs");
const utils_1 = require("./utils");
const path_1 = __importDefault(require("path"));
const fs = __importStar(require("fs/promises"));
const yaml = __importStar(require("js-yaml"));
const bson = __importStar(require("bson"));
const events_1 = require("events");
class BsonDate extends Date {
    constructor(value) {
        if (typeof value === 'string') {
            const number = Number(value);
            if (!Number.isNaN(number)) {
                value = number;
            }
        }
        super(value);
    }
    toString() {
        if (this.getTime() === 0) {
            return '0';
        }
        return this.toISOString();
    }
}
class GeneratorBase {
    constructor() {
        this.configDir = path_1.default.join(__dirname, '..', 'mongo-php-library', 'generator', 'config');
        if ('implicit' in yaml.DEFAULT_SCHEMA) {
            const implicit = yaml.DEFAULT_SCHEMA.implicit;
            const timestamp = implicit.find((type) => type.instanceOf === Date);
            if (timestamp) {
                timestamp.instanceOf = null;
                timestamp.predicate = (data) => {
                    return data instanceof Date && !(data instanceof BsonDate);
                };
            }
        }
    }
    async *listCategories(filterRegex) {
        for await (const folder of await fs.readdir(this.configDir, {
            withFileTypes: true,
        })) {
            if (folder.isDirectory() && filterRegex?.test(folder.name) !== false) {
                yield {
                    category: folder.name,
                    folder: path_1.default.join(folder.parentPath, folder.name),
                };
            }
        }
    }
    async *listSourceYAMLFiles(categoryRegex, operatorRegex) {
        for await (const { category, folder } of this.listCategories(categoryRegex)) {
            yield {
                category,
                operators: async function* () {
                    for await (const file of await fs.readdir(folder, {
                        withFileTypes: true,
                    })) {
                        if (file.isFile() &&
                            file.name.endsWith('.yaml') &&
                            operatorRegex?.test(file.name) !== false) {
                            const filePath = path_1.default.join(file.parentPath, file.name);
                            const content = await fs.readFile(filePath, 'utf8');
                            const parsed = yaml.load(content, GeneratorBase.loadOptions);
                            yield { yaml: parsed, path: filePath };
                        }
                    }
                },
            };
        }
    }
    async emitToFile(filePath) {
        await this.flushFile();
        this.outputStream = (0, fs_1.createWriteStream)(filePath, { encoding: 'utf8' });
    }
    async flushFile() {
        if (this.outputStream) {
            this.outputStream.end();
            await (0, events_1.once)(this.outputStream, 'close');
        }
    }
    emit(str) {
        (this.outputBuffer ?? this.outputStream ?? process.stdout).write(str);
    }
    toComment(str, docsUrl) {
        if (!str) {
            return '';
        }
        return [
            '',
            '',
            '/**',
            ...str
                .replace(/\*\//g, '*//*')
                .split('\n')
                .map((l) => l.trim())
                .filter((l) => l)
                .map((l) => ` * ${l}`),
            ...(docsUrl ? [` * @see {@link ${docsUrl}}`] : []),
            ' */',
            '',
        ].join('\n');
    }
    emitComment(str, docsUrl) {
        this.emit(this.toComment(str, docsUrl));
    }
    getOutputOf(fn) {
        this.outputBuffer = new utils_1.StringWriter();
        fn();
        const output = this.outputBuffer.toString();
        this.outputBuffer = undefined;
        return output;
    }
    async generate(categoryFilter, operatorFilter) {
        const categoryRegex = categoryFilter
            ? new RegExp(categoryFilter)
            : undefined;
        const operatorRegex = operatorFilter
            ? new RegExp(operatorFilter)
            : undefined;
        const files = this.listSourceYAMLFiles(categoryRegex, operatorRegex);
        await this.generateImpl(files);
        await this.flushFile();
    }
}
exports.GeneratorBase = GeneratorBase;
GeneratorBase.loadOptions = {
    schema: yaml.DEFAULT_SCHEMA.extend([
        new yaml.Type('!bson_utcdatetime', {
            kind: 'scalar',
            construct(data) {
                return new BsonDate(data);
            },
            instanceOf: BsonDate,
            represent(data) {
                if (data instanceof BsonDate) {
                    return data.toString();
                }
                throw new Error(`Expected Date, but got ${data.constructor.name}`);
            },
        }),
        new yaml.Type('!bson_objectId', {
            kind: 'scalar',
            construct(data) {
                return bson.ObjectId.createFromHexString(data);
            },
            predicate(data) {
                return data instanceof bson.ObjectId;
            },
            represent(data) {
                if (data instanceof bson.ObjectId) {
                    return data.toHexString();
                }
                throw new Error(`Expected bson.ObjectId, but got ${data.constructor.name}`);
            },
        }),
        new yaml.Type('!bson_uuid', {
            kind: 'scalar',
            construct(data) {
                return bson.UUID.createFromHexString(data);
            },
            predicate(data) {
                return data instanceof bson.UUID;
            },
            represent(data) {
                if (data instanceof bson.UUID) {
                    return data.toHexString();
                }
                throw new Error(`Expected bson.UUID, but got ${data.constructor.name}`);
            },
        }),
        new yaml.Type('!bson_regex', {
            kind: 'scalar',
            construct(data) {
                return new bson.BSONRegExp(data);
            },
            predicate(data) {
                return data instanceof bson.BSONRegExp && !data.options;
            },
            represent(data) {
                if (data instanceof bson.BSONRegExp) {
                    return data.pattern;
                }
                throw new Error(`Expected bson.BSONRegExp, but got ${data.constructor.name}`);
            },
        }),
        new yaml.Type('!bson_regex', {
            kind: 'sequence',
            construct([data, flags]) {
                return new bson.BSONRegExp(data, flags);
            },
            predicate(data) {
                return data instanceof bson.BSONRegExp && !!data.options;
            },
            represent(data) {
                if (data instanceof bson.BSONRegExp) {
                    return [data.pattern, data.options];
                }
                throw new Error(`Expected bson.BSONRegExp, but got ${data.constructor.name}`);
            },
        }),
        new yaml.Type('!bson_binary', {
            kind: 'scalar',
            construct(data) {
                return bson.Binary.createFromBase64(data);
            },
            predicate(data) {
                return data instanceof bson.Binary;
            },
            represent(data) {
                if (data instanceof bson.Binary) {
                    return data.toString('base64');
                }
                throw new Error(`Expected bson.Binary, but got ${data.constructor.name}`);
            },
        }),
        new yaml.Type('!bson_decimal128', {
            kind: 'scalar',
            construct(data) {
                return bson.Decimal128.fromString(data);
            },
            predicate(data) {
                return data instanceof bson.Decimal128;
            },
            represent(data) {
                if (data instanceof bson.Decimal128) {
                    return data.toString();
                }
                throw new Error(`Expected bson.Decimal128, but got ${data.constructor.name}`);
            },
        }),
        new yaml.Type('!bson_int64', {
            kind: 'scalar',
            construct(data) {
                return bson.Long.fromString(data);
            },
            predicate(data) {
                return data instanceof bson.Long;
            },
            represent(data) {
                if (data instanceof bson.Long) {
                    return data.toString();
                }
                throw new Error(`Expected bson.Long, but got ${data.constructor.name}`);
            },
        }),
    ]),
};
//# sourceMappingURL=generator.js.map